# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from jmb.filters.admin import AdvancedSearchModelAdmin
from mptt.admin import MPTTModelAdmin
from thx.core.admin import UserAdminMixin
from . import models


class RequiredFieldInline(admin.TabularInline):
    model = models.RequiredField
    fields = ('status', 'from_status', 'fields', 'ordering',)
    extra = 0


class StatusFlowInline(admin.TabularInline):
    model = models.StatusFlow
    fields = ('status', 'from_status', 'to_status', 'ordering',)
    extra = 0


class UsedFieldInline(admin.TabularInline):
    model = models.UsedField
    fields = ('status', 'from_status', 'action', 'fields', 'ordering',)
    extra = 0


class FilterInline(admin.TabularInline):
    model = models.Filter
    fields = ('status', 'action', 'codename', 'description', 'code', 'ordering',)
    readonly_fields = ('codename',)
    extra = 0


class UserRoleAdmin(AdvancedSearchModelAdmin, admin.ModelAdmin):
    search_fields = ('app_label', 'model_name',)
    list_display = ('id', 'username', 'email', 'status', 'app_label', 'model_name', 'name',)
    advanced_search_fields = (
        ('role__status',),
        ('role__name__icontains',),
        ('role__app_label__icontains', 'role__model_name__icontains',),
        ('user__username__icontains', 'user__email__icontains',),
    )
    list_display_links = ('app_label', 'model_name',)
    readonly_fields = ('role', 'user',)

    def username(self, obj):
        return obj.username
    username.admin_order_field = 'user__username'

    def email(self, obj):
        return obj.email
    email.admin_order_field = 'user__email'

    def status(self, obj):
        return obj.status
    status.admin_order_field = 'role__status'

    def app_label(self, obj):
        return obj.app_label
    app_label.admin_order_field = 'role__app_label'

    def model_name(self, obj):
        return obj.model_name
    model_name.admin_order_field = 'role__model_name'

    def name(self, obj):
        return obj.name
    name.admin_order_field = 'role__name'


class RoleAdmin(UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin):
    search_fields = ('name',)
    advanced_search_fields = (
        ('status',),
        ('name__icontains',),
        ('app_label__icontains', 'model_name__icontains',),
        ('users__email__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': ('name', ('app_label', 'model_name'), 'codename', 'users',)
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'app_label', 'model_name', 'name', 'ordering')
    list_display_links = ('name',)
    filter_horizontal = ('users',)
    readonly_fields = ('codename', 'creator', 'date_create', 'last_modifier', 'date_last_modify')
    autocomplete_fields = ('users',)
    inlines = (UsedFieldInline, RequiredFieldInline, StatusFlowInline, FilterInline)


class RequiredFieldAdmin(
    UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('role__app_label',)
    advanced_search_fields = (
        ('status',),
        ('role', 'from_status',),
        ('role__app_label__icontains', 'role__model_name__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': ('role', 'from_status', 'fields')
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'role', 'app_label', 'model_name', 'from_status', 'fields', 'ordering')
    list_display_links = ('role',)
    autocomplete_fields = ('role',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')


class StatusFlowAdmin(
    UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('role__app_label',)
    advanced_search_fields = (
        ('status',),
        ('role', 'from_status',),
        ('role__app_label__icontains', 'role__model_name__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': (
                'role',
                ('from_status', 'to_status',),
            )
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'role', 'app_label', 'model_name', 'from_status', 'to_status', 'ordering')
    list_display_links = ('role',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')


class UsedFieldAdmin(
    UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('role__app_label',)
    advanced_search_fields = (
        ('status',),
        ('role', 'from_status',),
        ('action',),
        ('role__app_label__icontains', 'role__model_name__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': ('role', 'from_status', 'action', 'fields')
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'role', 'app_label', 'model_name', 'from_status', 'action', 'fields', 'ordering')
    list_display_links = ('role',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')


class FilterAdmin(
    UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('name',)
    advanced_search_fields = (
        ('status',),
        ('role', 'action',),
        ('role__app_label__icontains', 'role__model_name__icontains',),
    )
    list_filter = ('status',)
    fieldsets = (
        (_('general information'), {
            'fields': (
                'codename', 'role', 'action',
                'description', 'code',
            )
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status')
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = ('id', 'status', 'role', 'app_label', 'model_name', 'action', 'ordering')
    list_display_links = ('role',)
    readonly_fields = ('codename', 'creator', 'date_create', 'last_modifier', 'date_last_modify')


class MenuAdmin(
    MPTTModelAdmin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    search_fields = ('title',)
    advanced_search_fields = (
        ('title__icontains',),
        ('parent__title__icontains',),
    )
    list_filter = ('status', 'language_code')
    fieldsets = (
        (_('general information'), {
            'fields': (
                'parent',
                'title',
                'language_code', 'router',
                'roles',
                'info',
            ),
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', 'status',),
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
        }),
    )
    list_display = ('id', 'status', 'language_code', 'parent', 'title', 'role_list', 'ordering',)
    list_display_links = ('title',)
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify',)
    autocomplete_fields = ('parent', 'roles',)


admin.site.register(models.UserRole, UserRoleAdmin)
admin.site.register(models.Role, RoleAdmin)
admin.site.register(models.RequiredField, RequiredFieldAdmin)
admin.site.register(models.StatusFlow, StatusFlowAdmin)
admin.site.register(models.UsedField, UsedFieldAdmin)
admin.site.register(models.Filter, FilterAdmin)
admin.site.register(models.Menu, MenuAdmin)
